#roombaWiiMote
#Control a roomba bot with a wii remote!


# Wii remote code based on code by The Raspberry Pi Guy. Work based on some of Matt Hawkins's!


import itertools
import json
import re
import requests
import serial
import cwiid		#wii remote library
import time
import os
import pigpio		#for pigpio to work, you need to run "sudo pigpiod" first, this starts the daemeon
import struct
 
import RPi.GPIO as GPIO
pi = pigpio.pi()

import subprocess




DD =  18  #device detect
button_delay = 0.1	#for wiimote
HEAD = 17	#servo pin number
NECK = 22	#servo pin number
HEADMIN = 1100		#servo pulse width limits
HEADMAX = 2100
HEADCENTER = 1600
NECKMIN = 1100
NECKMAX = 2100
NECKCENTER = 1600

#driving variables for roomba
velocity = 0
radius = 0
forwardIntent = False	#used to make the roomba turn only slightly if it is already going forward, vs tight turns if standing still
oldVel = 0		# to prevent continuely sending data to the roomba if it doesn't change
oldRad = 0
headCon = False		#toggle head servo control with A button
 
GPIO.setmode(GPIO.BCM)
GPIO.setup(DD, GPIO.OUT)

#Wake up the roomba
GPIO.output(18, False)
time.sleep(1)
GPIO.output(18, True)
time.sleep(2)

#start serial with roomba
ser = serial.Serial(port='/dev/ttyAMA0',baudrate=57600, timeout=1)
ser.write(serial.to_bytes("\x80"))

# everything has to be sent as hex code bytes. So using the SCI programmer's 
# guide to get the decimal codes, convert to hex
# and put them in here with \x in front of each 2 digit byte to denote hex...

time.sleep(0.1)
ser.write(serial.to_bytes("\x82"))
time.sleep(0.2)
ser.write(serial.to_bytes("\x84"))

#light RED LED
ser.write(serial.to_bytes("\x8b\x19\xFF\x80"))

#play a song
time.sleep(2)
# make a song with some tones
ser.write(serial.to_bytes("\x8c\x00\x02\x60\x40\x60\x40"))
#now play it.
ser.write(serial.to_bytes("\x8d\x00"))
time.sleep(1)


def drive():                   #end driving commands to roomba
	ser.write(serial.to_bytes("\x89"))
	ser.write(serial.to_bytes(struct.pack('>h', velocity)))
	ser.write(serial.to_bytes(struct.pack('>h', radius)))


#connect wii remote
print 'Please press buttons 1 + 2 on your Wiimote now ...'

# This code attempts to connect to your Wiimote and if it fails the program quits
try:
	wii=cwiid.Wiimote()
except RuntimeError:
	print "Cannot connect to your Wiimote. Run again and make sure you are holding buttons 1 + 2!"
	quit()

print 'Wiimote connection established!\n'
print 'Go ahead and press some buttons\n'
print 'Press PLUS and MINUS together to disconnect and quit.\n'

time.sleep(3)

wii.rpt_mode = cwiid.RPT_BTN

while True:

	buttons = wii.state['buttons']
	forwardIntent = False
	velocity = 0
	radius = -32768

  # Detects whether + and - are held down and if they are it quits the program
	if (buttons - cwiid.BTN_PLUS - cwiid.BTN_MINUS == 0):
		print '\nClosing connection ...'
    # NOTE: This is how you RUMBLE the Wiimote
		wii.rumble = 1
		time.sleep(1)
		wii.rumble = 0
		exit(wii)

  # The following code detects whether any of the Wiimotes buttons have been pressed and then prints a statement to the screen!
	if (buttons & cwiid.BTN_UP):
		if (headCon):
			pi.set_servo_pulsewidth(NECK, NECKCENTER)
			pi.set_servo_pulsewidth(HEAD, HEADCENTER)
		else:
			print 'Up pressed'
			#pi.set_servo_pulsewidth(NECK, 1500)
			velocity = 500
			radius = -32768
			forwardIntent = True
			time.sleep(button_delay)
	
	if (buttons & cwiid.BTN_LEFT):
		if (headCon):
			pi.set_servo_pulsewidth(NECK, NECKMIN)
		else:
			print 'Left pressed'
			#pi.set_servo_pulsewidth(NECK, 1000)
			if (forwardIntent == True):
				velocity = 500
				radius = 200
			else:
				velocity = 300
				radius = 20

			time.sleep(button_delay)

	if(buttons & cwiid.BTN_RIGHT):
		if (headCon):
			pi.set_servo_pulsewidth(NECK, NECKMAX)
		else:
			print 'Right pressed'
			#pi.set_servo_pulsewidth(NECK, 2000)
			if (forwardIntent == True):
				velocity = 500
				radius = -200
			else:
				velocity = 300
				radius = -20

			time.sleep(button_delay)

	if (buttons & cwiid.BTN_DOWN):
		print 'Down pressed'
		velocity = -300
		radius = -32768			
		time.sleep(button_delay)

	if (buttons & cwiid.BTN_1):
		print 'Button 1 pressed'
		subprocess.Popen(["python", 'lightbartest.py'])	#start lightbar code
		time.sleep(button_delay)

	if (buttons & cwiid.BTN_2):
		print 'Button 2 pressed'
		subprocess.Popen(["python", 'flash.py'])	#start flash lightbar code
		time.sleep(button_delay)

	if (buttons & cwiid.BTN_A):
		print 'Button A pressed'
		if (headCon):
			headCon = False
		else:
			headCon = True
		time.sleep(1)

	if (buttons & cwiid.BTN_B):
		print 'Button B pressed'
		wii.rpt_mode = cwiid.RPT_BTN | cwiid.RPT_ACC
		check = 0
		while check == 0:
			print(wii.state['acc'])		#control head servos with accelerometer data
			time.sleep(0.0001)
			accelNeck = (wii.state['acc'][0] * 18) - 700
			accelHead = (wii.state['acc'][1] * -18) + 3800
			if (accelNeck > NECKMAX):
				accelNeck = NECKMAX
			elif (accelNeck < NECKMIN):
				accelNeck = NECKMIN
			
			if (accelHead > HEADMAX):
				accelHead = HEADMAX
			elif (accelHead < HEADMIN):
				accelHead = HEADMIN
				
			pi.set_servo_pulsewidth(NECK, accelNeck)
			pi.set_servo_pulsewidth(HEAD, accelHead)
			check = (buttons & cwiid.BTN_B)
		time.sleep(button_delay)
		

	if (buttons & cwiid.BTN_HOME):
		wii.rpt_mode = cwiid.RPT_BTN | cwiid.RPT_ACC
		check = 0
		while check == 0:
			print(wii.state['acc'])
			time.sleep(0.01)
			check = (buttons & cwiid.BTN_HOME)
			time.sleep(button_delay)

	if (buttons & cwiid.BTN_MINUS):
		print 'Minus Button pressed'
		time.sleep(button_delay)

	if (buttons & cwiid.BTN_PLUS):
		print 'Plus Button pressed'
		velocity = 500
		radius = -32768
		drive()
		time.sleep(7)
		time.sleep(button_delay)
	
	if ((oldVel != velocity) or (oldRad != radius)):	#send data to roomba
		oldVel = velocity
		oldRad = radius
		print (velocity, radius)
		drive()


	